//-*-c++-*------------------------------------------------------------
//
// File name : bioArithAnd.cc
// Author :    Michel Bierlaire and Mamy Fetiarison
// Date :      Tue May  5 10:36:07 2009
//
//--------------------------------------------------------------------

#include <sstream>

#include "patDisplay.h"
#include "patErrNullPointer.h"
#include "patErrMiscError.h"

#include "bioArithAnd.h"
#include "bioArithConstant.h"

bioArithAnd::bioArithAnd(bioExpression* par,
                         bioExpression* left, 
                         bioExpression* right) : bioExpression(par,left,right)
{
  DEBUG_MESSAGE("-----> Create AND operator") ;
}

bioArithAnd::~bioArithAnd() {}

bioExpression::bioOperatorType bioArithAnd::getOperatorType() const {
  return bioExpression::BINARY_OP ;
}

patString bioArithAnd::getOperatorName() const {
  return ("&&") ;
}

patReal bioArithAnd::getValue(patError*& err) const  {
  if (leftChild == NULL || rightChild == NULL) {
    err = new patErrNullPointer("bioExpression") ;
    WARNING(err->describe()) ;
    return patReal();
  }
  
  patReal left = leftChild->getValue(err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return patReal();
  }
  patReal right = rightChild->getValue(err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return patReal();
  }
  patReal result = left && right ;

  return result ;
}

bioExpression* bioArithAnd::getDerivative(patULong aLiteralId, 
					  patError*& err) const {
  if (dependsOf(aLiteralId)) {
    err = new patErrMiscError("No derivative for boolean functions") ;
    WARNING(err->describe()) ;
    return NULL ;
  }
  else {
    bioExpression* result = new bioArithConstant(NULL,0.0) ;
    return result ;
  }
}

bioArithAnd* bioArithAnd::getDeepCopy(patError*& err) {
  bioExpression* leftClone(NULL) ;
  bioExpression* rightClone(NULL) ;
  if (leftChild != NULL) {
    leftClone = leftChild->getDeepCopy(err) ;
  }
  if (rightChild != NULL) {
    rightClone = rightChild->getDeepCopy(err) ;
  }

  bioArithAnd* theNode = 
    new bioArithAnd(NULL,leftClone,rightClone) ;
  leftClone->setParent(theNode) ;
  rightClone->setParent(theNode) ;
  return theNode ;
}


patString bioArithAnd::getCppCode(patString prefix, patError*& err) {
  if (codeMustBeGenerated) {
  if (leftChild == NULL || rightChild == NULL) {
    err = new patErrNullPointer("bioExpression") ;
    WARNING(err->describe()) ;
    return patString();
  }
  
  patBoolean lSimple = leftChild->isSimple() ;
  patBoolean rSimple = rightChild->isSimple() ;
  patString vLeft ;
  if (lSimple) {
    vLeft = leftChild->getSimpleCppCode(err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return patString();
    }
  }
  else {
    vLeft = leftChild->getCppCode(prefix, err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return patString();
    }
  }
  patString vRight ;
  if (rSimple) {
    vRight = rightChild->getSimpleCppCode(err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return patString();
    }
  }
  else {
    vRight = rightChild->getCppCode(prefix, err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return patString();
    }
  }
  stringstream str ;
  if (!lSimple) {
    str << vLeft << endl ;
  }
  if (!rSimple) {
    str << vRight << endl ;
  }
  str << getCppResultDeclaration(prefix) << " = (" ;
  if (lSimple) {
    str << vLeft ;
  }
  else {
    str << leftChild->getCppResult(prefix) ;
  }
  str << ") * (" ;
  if (rSimple) {
    str << vRight;    
  }
  else {
    str << rightChild->getCppResult(prefix) ;
  }
  str << ");" << endl ;
  //  str << "DEBUG_MESSAGE(\"" << getCppResult(prefix) << "= \" << " << getCppResult(prefix) << ");" << endl ;
  codeMustBeGenerated = patFALSE ;
  return patString(str.str()) ;
  }
  else {
    return patString() ;
  }
}

patString bioArithAnd::getSimpleCppCode(patError*& err) {
  if (leftChild == NULL || rightChild == NULL) {
    err = new patErrNullPointer("bioExpression") ;
    WARNING(err->describe()) ;
    return patString();
  }
  
  patString vLeft = leftChild->getSimpleCppCode(err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return patString();
  }
  DEBUG_MESSAGE("vLeft = " << vLeft) ;
  patString vRight = rightChild->getSimpleCppCode(err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return patString();
  }
  DEBUG_MESSAGE("vRight = " << vRight) ;
  stringstream str ;
  str << "(" << vLeft << ")*(" << vRight << ")" ;
  return patString(str.str()) ;
}


patString bioArithAnd::getExpressionString() const {
  stringstream str ;
  str << '&' ;
  if (leftChild != NULL) {
    str << '{' << leftChild->getExpressionString() << '}' ;
  }
  if (rightChild != NULL) {
    str << '{' << rightChild->getExpressionString() << '}' ;
  }
  return patString(str.str()) ;

}

