//-*-c++-*------------------------------------------------------------
//
// File name : bioArithGreater.cc
// Author :    Michel Bierlaire and Mamy Fetiarison
// Date :      Tue May  5 11:25:34 2009
//
//--------------------------------------------------------------------

#include "patDisplay.h"
#include "patErrNullPointer.h"
#include "patErrMiscError.h"
#include "bioArithGreater.h"
#include "bioArithConstant.h"

#include "bioLiteralRepository.h"
bioArithGreater::bioArithGreater(bioExpression* par,
                                 bioExpression* left, 
                                 bioExpression* right) : bioExpression(par,left,right)
{
}

bioArithGreater::~bioArithGreater() {}

bioExpression::bioOperatorType bioArithGreater::getOperatorType() const {
  return bioExpression::BINARY_OP ;
}

patString bioArithGreater::getOperatorName() const {
  return (">") ;
}


patReal bioArithGreater::getValue(patError*& err) const  {
  if (leftChild == NULL || rightChild == NULL) {
    err = new patErrNullPointer("bioExpression") ;
    WARNING(err->describe()) ;
    return patReal();
  }

  patReal result = (leftChild->getValue(err) > rightChild->getValue(err)) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return patReal();
  }

  return result ;
}


bioExpression* bioArithGreater::getDerivative(patULong aLiteralId, 
					      patError*& err) const {
  if (dependsOf(aLiteralId)) {
    stringstream str ;
    patString theParam = bioLiteralRepository::the()->getName(aLiteralId,err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return NULL ;
    }
    str << "Differentiation of a boolean function with respect to " << theParam << ": " << *this ;
    err = new patErrMiscError(str.str()) ;
    WARNING(err->describe()) ;
    return NULL ;
  }
  else {
    bioExpression* result = new bioArithConstant(NULL,0.0) ;
    return result ;
  }
}


bioArithGreater* bioArithGreater::getDeepCopy(patError*& err) {
  bioExpression* leftClone(NULL) ;
  bioExpression* rightClone(NULL) ;
  if (leftChild != NULL) {
    leftClone = leftChild->getDeepCopy(err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return NULL ;
    }
    if (leftClone == NULL) {
      err = new patErrNullPointer("bioExpression") ;
      WARNING(err->describe()) ;
      return NULL ;
    }
  }
  if (rightChild != NULL) {
    rightClone = rightChild->getDeepCopy(err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return NULL ;
    }
    if (rightClone == NULL) {
      err = new patErrNullPointer("bioExpression") ;
      WARNING(err->describe()) ;
      return NULL ;
    }
  }

  bioArithGreater* theNode = 
    new bioArithGreater(NULL,leftClone,rightClone) ;
  leftClone->setParent(theNode) ;
  rightClone->setParent(theNode) ;
  return theNode ;
}


patString bioArithGreater::getCppCode(patString prefix, patError*& err) {
  if (codeMustBeGenerated) {
  if (leftChild == NULL || rightChild == NULL) {
    err = new patErrNullPointer("bioExpression") ;
    WARNING(err->describe()) ;
    return patString();
  }
  
  patBoolean lSimple = leftChild->isSimple() ;
  patBoolean rSimple = rightChild->isSimple() ;
  patString vLeft ;
  if (lSimple) {
    vLeft = leftChild->getSimpleCppCode(err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return patString();
    }
  }
  else {
    vLeft = leftChild->getCppCode(prefix, err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return patString();
    }
  }
  patString vRight ;
  if (rSimple) {
    vRight = rightChild->getSimpleCppCode(err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return patString();
    }
  }
  else {
    vRight = rightChild->getCppCode(prefix, err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return patString();
    }
  }
  stringstream str ;
  if (!lSimple) {
    str << vLeft << endl ;
  }
  if (!rSimple) {
    str << vRight << endl ;
  }
  str << getCppResultDeclaration(prefix) << " = (" ;
  if (lSimple) {
    str << "(" << vLeft << ")" ;
  }
  else {
    str << leftChild->getCppResult(prefix) ;
  }
  str << ">" ;
  if (rSimple) {
    str << "(" << vRight << ")" ;    
  }
  else {
    str << rightChild->getCppResult(prefix) ;
  }
  str << ")?1:0;" << endl ;
  //  str << "DEBUG_MESSAGE(\"" << getCppResult(prefix) << "= \" << " << getCppResult(prefix) << ");" << endl ;
  codeMustBeGenerated = patFALSE ;
  return patString(str.str()) ;
  }
  else {
    return patString() ;
  }
}

patString bioArithGreater::getSimpleCppCode(patError*& err) {
  if (leftChild == NULL || rightChild == NULL) {
    err = new patErrNullPointer("bioExpression") ;
    WARNING(err->describe()) ;
    return patString();
  }
  
  patString vLeft = leftChild->getSimpleCppCode(err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return patString();
  }
  patString vRight = rightChild->getSimpleCppCode(err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return patString();
  }
  stringstream str ;
  str << "(((" << vLeft << ") > (" << vRight << "))?1:0)" ;
  return patString(str.str()) ;
}





patString bioArithGreater::getExpressionString() const {
  stringstream str ;
  str << '>o' ;
  if (leftChild != NULL) {
    str << '{' << leftChild->getExpressionString() << '}' ;
  }
  if (rightChild != NULL) {
    str << '{' << rightChild->getExpressionString() << '}' ;
  }
  return patString(str.str()) ;

}
