//-*-c++-*------------------------------------------------------------
//
// File name : bioLiteral.cc
// Author :    Michel Bierlaire and Mamy Fetiarison
// Date :      Mon Apr 27 17:23:57 2009
//
//--------------------------------------------------------------------

#include <sstream>
#include "patErrMiscError.h"
#include "patDisplay.h"
#include "bioArithLiteral.h"
#include "bioArithConstant.h"
#include "bioLiteralRepository.h"
#include "bioLiteralValues.h"

// [To check]
bioArithLiteral::bioArithLiteral(bioExpression* par, patULong aLiteralId)
  : bioExpression(par,NULL,NULL), theLiteralId(aLiteralId) {
  patError* err(NULL) ;
  theName = bioLiteralRepository::the()->getName(theLiteralId,err) ;
}

patString bioArithLiteral::getOperatorName() const {
  patError* err(NULL) ;
  return theName ;
}


bioExpression::bioOperatorType bioArithLiteral::getOperatorType() const {
  return bioExpression::LITERALS_OP ;
}

patString bioArithLiteral::getExpression(patError*& err) const {
  return bioExpression::getExpression(err);
}

patReal bioArithLiteral::getValue(patError*& err) const {
  patReal r = bioLiteralValues::the()->getValue(theName,err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return patReal() ;
  }
  return r ;
}

bioExpression* bioArithLiteral::getDerivative(patULong aLiteralId, patError*& err) const {
  if (aLiteralId == theLiteralId) {
    bioExpression* theExpr = new bioArithConstant(NULL,1) ;
    return theExpr ;
  }
  else {
    bioExpression* theExpr = new bioArithConstant(NULL,0) ;
    return theExpr ;
  }
}

bioExpression* bioArithLiteral::getDeepCopy(patError*& err) {
  bioArithLiteral* theNewLiteral = new bioArithLiteral(NULL, theLiteralId) ;
  theNewLiteral->leftChild = NULL ;
  theNewLiteral->rightChild = NULL ;
  theNewLiteral->parent = NULL ;
  return theNewLiteral ;
}

patString bioArithLiteral::getSimpleCppCode(patError*& err) {
  stringstream str  ;
  str << bioLiteralRepository::the()->getCppCode(theLiteralId,err);
  return patString(str.str()) ;
}

patString bioArithLiteral::getCppCode(patString prefix, patError*& err) {
  if (codeMustBeGenerated) {
    stringstream str  ;
    str << getCppResultDeclaration(prefix) << " = "  << bioLiteralRepository::the()->getCppCode(theLiteralId,err) << " ; " << endl ;
    //    str << "DEBUG_MESSAGE(\"" << getCppResult(prefix) << "= \" << " << getCppResult(prefix) << ");" << endl ;
  codeMustBeGenerated = patFALSE ;
    return patString(str.str()) ;
  }
  else {
    return patString() ;
  }
}


patBoolean bioArithLiteral::dependsOf(patULong aLiteralId) const {
  patError* err(NULL) ;
  if (aLiteralId == theLiteralId) {
    return patTRUE ;
  }
  else {
    return patFALSE ;
  }
}

patULong bioArithLiteral::getNumberOfOperations() const {
  return 0 ;
}
patString bioArithLiteral::getExpressionString() const {
  stringstream str ;
  str << theLiteralId ;
  return patString(str.str()) ;

}
