//-*-c++-*------------------------------------------------------------
//
// File name : bioArithUnaryMinus.cc
// Author :    Michel Bierlaire and Mamy Fetiarison
// Date :      Fri May 15 09:41:41 2009
//
//--------------------------------------------------------------------

#include "bioArithUnaryMinus.h"
#include "patErrNullPointer.h"
#include "patErrMiscError.h"
#include "patDisplay.h"

bioArithUnaryMinus::bioArithUnaryMinus(bioExpression* par,
                                       bioExpression* left) 
  : bioExpression(par,left,NULL) {

}

bioArithUnaryMinus::~bioArithUnaryMinus() {
}

bioExpression::bioOperatorType bioArithUnaryMinus::getOperatorType() const {
  return bioExpression::UNARY_OP ;
}

patString bioArithUnaryMinus::getOperatorName() const {
  return("-") ;
}

patReal bioArithUnaryMinus::getValue(patError*& err) const {
  
  if (leftChild == NULL) {
    err = new patErrNullPointer("bioExpression") ;
    WARNING(err->describe()) ;
    return patReal();
  }
  
  patReal result = leftChild->getValue(err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return patReal();
  }

  return -result ;
}

bioExpression* bioArithUnaryMinus::getDerivative(patULong aLiteralId, 
						 patError*& err) const {
  if (leftChild == NULL) {
    err = new patErrNullPointer("bioExpression") ;
    WARNING(err->describe()) ;
    return NULL;
  }
  
  bioExpression* leftResult = leftChild->getDerivative(aLiteralId,err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return NULL;
  }

  bioExpression* result = new bioArithUnaryMinus(NULL,leftResult) ;
  return result ;
}

bioArithUnaryMinus* bioArithUnaryMinus::getDeepCopy(patError*& err) {
  bioExpression* leftClone(NULL) ;
  bioExpression* rightClone(NULL) ;
  if (leftChild != NULL) {
    leftClone = leftChild->getDeepCopy(err) ;
  }
  if (rightChild != NULL) {
    rightClone = rightChild->getDeepCopy(err) ;
  }

  bioArithUnaryMinus* theNode = 
    new bioArithUnaryMinus(NULL,leftClone) ;
  leftClone->setParent(theNode) ;
  return theNode ;
}

patString bioArithUnaryMinus::getCppCode(patString prefix,patError*& err) {
  if (codeMustBeGenerated) {
    if (leftChild == NULL) {
      err = new patErrNullPointer("bioExpression") ;
      WARNING(err->describe()) ;
      return patString();
    }
    patString vLeft = leftChild->getCppCode(prefix, err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return patString();
    }
    stringstream str ;
    str << vLeft << endl ;
    str << getCppResultDeclaration(prefix) << " = - ( " << leftChild->getCppResult(prefix) << "  ) ;" << endl ;
    //    str << "DEBUG_MESSAGE(\"" << getCppResult(prefix) << "= \" << " << getCppResult(prefix) << ")" ;
    codeMustBeGenerated = patFALSE ;
    return patString(str.str()) ;
  }
  else {
    return patString() ;
  }
}

patString bioArithUnaryMinus::getSimpleCppCode(patError*& err) {
  if (leftChild == NULL) {
    err = new patErrNullPointer("bioExpression") ;
    WARNING(err->describe()) ;
    return patString();
  }
  patString vLeft = leftChild->getSimpleCppCode(err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return patString();
  }
  stringstream str ;
  str << "-(" << vLeft << ")" ;
  return patString(str.str()) ;
}



patString bioArithUnaryMinus::getExpressionString() const {
  stringstream str ;
  str << '$M' ;
  if (leftChild != NULL) {
    str << '{' << leftChild->getExpressionString() << '}' ;
  }
  return patString(str.str()) ;
}
