//-*-c++-*------------------------------------------------------------
//
// File name : bioGenerateStatistics.h
// Author :    Michel Bierlaire and Mamy Fetiarison
// Date :      Sat Dec 19 17:14:24 2009
//
//--------------------------------------------------------------------

#include "patLargeNumbers.h"
#include "patMath.h"
#include "patErrMiscError.h"
#include "patDisplay.h"
#include "bioGenerateStatistics.h"
#include "bioLiteralRepository.h"
#include "bioExpressionRepository.h"
#include "bioVersion.h"
#include "bioExpression.h"
#include "bioIteratorInfoRepository.h"
#include "bioArithMergedSum.h"
#include "bioCountOperations.h"
#include "bioParameters.h"

bioGenerateStatistics::bioGenerateStatistics(map<patString,patULong>* lit, 
							     map<patString,bioExpression*>* statistics,
							     patString f):
  theStatistics(statistics), 
  literals(lit),
  mainFileName(f) {

}

void bioGenerateStatistics::generate(patError*& err) {

  vector<patULong> theIds = 
    bioLiteralRepository::the()->getBetaIds(patFALSE,err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return ;
  }

  // Simplify the expressions

  // LATER...


  stringstream str ;
  ofstream mk("files.mk",ios::app) ;
  mk << mainFileName << " " ;
  mk.close() ;
  ofstream f(mainFileName.c_str()) ;
  writeHeaders(f) ;
  f << "#include \"bioMyObjects.h\"" << endl ;
  f << "" << endl ;
  f << "void myBioStatistics::bio__generateStatistics(bioSample* sample, patError*& err) {" << endl ;
  
  f << "  trVector theBetas = bioLiteralRepository::the()->getBetaValues() ; " << endl ;
  f << " trVector* __beta = &theBetas ;" << endl ;
  f << "   bioIteratorSpan __theThreadSpan ;" << endl ;
  //  f << "   patVariables* __x ;" << endl ;
  f << "   vector< vector<patReal> >* __draw ;" << endl ;
  patString statPrefix("sim") ;
  if (theStatistics != NULL) {
    f << "      pair<patString,patReal> p ;" << endl ;
    for (map<patString,bioExpression*>::iterator i = theStatistics->begin() ;
	 i != theStatistics->end() ;
	 ++i) {
      f << "      p.first = \"" << i->first << "\" ;" << endl ;
      bioExpression* theExpression = i->second ;
      if (theExpression->isSimple()){
	f << "    p.second = " << theExpression->getSimpleCppCode(err) << " ;" << endl ;
	if (err != NULL) {
	  WARNING(err->describe()) ;
	}
      }
      else {
	f << theExpression->getCppCode(statPrefix,err) << endl ;
	if (err != NULL) {
	  WARNING(err->describe()) ;
	}
	f << "   p.second = "<<theExpression->getCppResult(statPrefix) <<" ;" << endl ;
      }
      f << "      statistics.push_back(p) ;" << endl ;
      ofstream gslhfile("bioGslBiogeme.h",ios::app) ;
      theExpression->generateGslFunctions(gslhfile,statPrefix,err) ;
      if (err != NULL) {
	WARNING(err->describe()) ;
	return ;
      }
      gslhfile.close() ;
    }

    f << "    return ;" << endl ;
  }
  f << "}"  << endl;
  
  f.close() ;
}




void bioGenerateStatistics::writeHeaders(ofstream& f) {
  patAbsTime now ;
  now.setTimeOfDay() ;
  f << "// This file has automatically been generated." << endl ;
  f << "// " << now.getTimeString(patTsfFULL) << endl ;
  f << "// " << bioVersion::the()->getCopyright() << endl ;
  f << endl ;
  f << "// " << bioVersion::the()->getVersionInfoDate() << endl ;
  f << "// " << bioVersion::the()->getVersionInfoAuthor() << endl ;
  f << endl ;

  f << " #define NUMTHREADS 1" << endl ;
  f << " " << endl ;


  f << "#include <assert.h>" << endl ;
  f << "#include \"bioSample.h\"" << endl ;
  f << "#include \"bioGaussHermite.h\"" << endl ;
  f << "#include \"patFileNames.h\"" << endl ;
  f << "#include \"bioVersion.h\"" << endl ;
  f << "#include \"bioIterates.h\"" << endl ;
  f << "#include \"bioIteratorInfoRepository.h\"" << endl ;
  f << "#include \"bioLiteralRepository.h\"" << endl ;
  f << "#include \"bioGslParams.h\"" << endl ;
  f << " " << endl ;
  f << "   #include \"bioMetaIterator.h\"" << endl ;
  f << "   #include \"bioDrawIterator.h\"" << endl ;
  f << "   #include \"bioRandomDraws.h\"" << endl ;
  f << "   #include \"bioRowIterator.h\"" << endl ;
  f << "   #include \"bioMain.h\"" << endl ;
  f << "   #include \"patMath.h\"" << endl ;
  f << "   #include \"bioGslBiogeme.h\"" << endl ;
  f << "   #include \"bioStatistics.h\"" << endl ;
  f << "   #include \"patVariables.h\"" << endl ;
  f << " " << endl ;
  //  f << " #define DEBUG_MESSAGE(message)" << endl ;
  //f << " #define DEBUG_MESSAGE(message) std::cout << message << endl ;" << endl ;

}


