//-*-c++-*------------------------------------------------------------
//
// File name : bioLiteralRepository.h
// Author :    Michel Bierlaire and Mamy Fetiarison
// Date :      Mon Apr 27 16:06:48 2009
//
//--------------------------------------------------------------------

#ifndef bioLiteralRepository_h
#define bioLiteralRepository_h

#include <map>
#include "bioLiteral.h"
#include "bioVariable.h"
#include "bioRandomVariable.h"
#include "bioFixedParameter.h"
#include "patError.h"
#include "patVariables.h"

/*!
Class gathering all literals, keeping track of their ID,
guaranteeing their uniqueness. Literals can be access by name or ID.
*/

class bioLiteralRepository {

  friend ostream& operator<<(ostream &str, const bioLiteralRepository& x) ;  
public:
  static bioLiteralRepository* the() ;
  patULong addVariable(patString theName, patULong colId, patError*& err) ;
  patULong addRandomVariable(patString theName, patError*& err) ;
  patULong addFixedParameter(patString theName, patReal val, patError*& err) ;
  patULong addFixedParameter(patString theName, patReal val, patReal lb, patReal up, patBoolean fixed, patString description, patError*& err) ;
  patULong addIntermediateValue(patError*& err) ;
  patULong addFixedParameter(bioFixedParameter p) ;
  patULong getLiteralId(patString theName, patError*& err) const ;
  patULong getLiteralId(const bioLiteral* theLiteral, patError*& err) const ;
  patString getName(patULong theId, patError*& err) ;
  patString getCppCode(patULong theId, patError*& err) ;
  void computeIds() ;
  map<patString,patULong>* getIdsOfVariables() ;
  map<patString,patULong>* getIdsOfRandomVariables() ;
  map<patString,patULong>* getIdsOfParameters() ;
  patULong getNumberOfVariables() const ;
  patULong getNumberOfRandomVariables() const ;
  patULong getNumberOfParameters() const ;
  patULong getNumberOfEstimatedParameters() ;
  // If all is patFALSE, only non fixed variables are returned
  patVariables getBetaValues(patBoolean all=patTRUE) const ;
  patVariables getLowerBounds() const ;
  patVariables getUpperBounds() const ;
  void setBetaValues(patVariables b, patError*& err) ;
  // return the literal ids of the beta in the right order. If all is
  // patFALSE, only non fixed betas are returned
  vector<patULong> getBetaIds(patBoolean all, patError*& err) const ;
  patString getBetaName(patULong betaId, patError*& err) ;
  bioFixedParameter* getBetaParameter(patULong literalId, patError*& err) ;
  void setFlag(patULong id) ;
  void unsetFlag(patULong id) ;
  patBoolean isFlagSet(patULong id) const ;
  void resetAllFlags() ;
  patString printFixedParameters(patBoolean estimation=patFALSE) const ;
private:
  bioLiteralRepository() ;
  vector<bioVariable> listOfVariables ;
  vector<bioFixedParameter> listOfFixedParameters ;
  vector<bioRandomVariable> listOfRandomVariables ;
  // Intermediate variables are not defined by the user. They are
  // created by biogeme to simplify expressions
  vector<patString> listOfIntermediateVariables ;
  vector<patString> listOfLiterals ;
  map<patULong,patBoolean> flags ;
  map<patString,patULong> listOrganizedByNames ;
  map<patString,patULong> idsOfVariables ;
  map<patString,patULong> idsOfRandomVariables ;
  map<patString,patULong> idsOfParameters ;

  // First: index in the vector listOfFixedParameters
  // Second: index in the set of decision variables for the optimization problem
  map<patULong,patULong> parametersToBeEstimated ;

};

#endif
