//-*-c++-*------------------------------------------------------------
//
// File name : bioPythonWrapper.cc
// Author :    Michel Bierlaire and Mamy Fetiarison
// Date :      Fri Jul 17 14:02:39 2009
//
//--------------------------------------------------------------------

#include <fstream>
#include "bioSample.h"
#include "patFileNames.h"
#include "bioPythonWrapper.h"
#include "patErrMiscError.h"
#include "bioParameters.h"
#include "bioIteratorInfoRepository.h"

bioPythonWrapper::bioPythonWrapper() {
  WARNING("The parameters should be initialized fron the bioParameters object. Here the default values are used") ;
}
bioPythonWrapper::~bioPythonWrapper() {

}


patReal bioPythonWrapper::getFunction(trVector* x,
				    patBoolean* success,
				    patError*& err) {
  err = new patErrMiscError("This function should never be called. It must be overloaded") ;
  WARNING(err->describe()) ;
  return patReal() ;
}

trVector* bioPythonWrapper::getGradient(trVector* x,
				      trVector* grad,
				      patBoolean* success,
				      patError*& err) {
  err = new patErrMiscError("This function should never be called. It must be overloaded") ;
  WARNING(err->describe()) ;
  return NULL ;
}

patReal bioPythonWrapper::getFunctionAndGradient(trVector* x,
					       trVector* grad,
					       patBoolean* success,
					       patError*& err){
  err = new patErrMiscError("This function should never be called. It must be overloaded") ;
  WARNING(err->describe()) ;
  return patReal() ;
}

trHessian* bioPythonWrapper::bioPythonWrapper::computeHessian(patVariables* x,
							  trHessian& hessian,
							  patBoolean* success,
							  patError*& err) {
  err = new patErrMiscError("This function should never be called. It must be overloaded") ;
  WARNING(err->describe()) ;
  return NULL ;

}
 
trHessian* bioPythonWrapper::getCheapHessian(trVector* x,
					   trHessian& hessian,
					   patBoolean* success,
					   patError*& err) {
  err = new patErrMiscError("This function should never be called. It must be overloaded") ;
  WARNING(err->describe()) ;
  return NULL ;
  
}

patBoolean bioPythonWrapper::isCheapHessianAvailable() {
  return patFALSE ;
}

trVector bioPythonWrapper::getHessianTimesVector(trVector* x,
					       const trVector* v,
					       patBoolean* success,
					       patError*& err) {
  err = new patErrMiscError("This function should never be called. It must be overloaded") ;
  WARNING(err->describe()) ;
  return trVector() ;
}

patBoolean bioPythonWrapper::isGradientAvailable(patError*& err) const {
  return patFALSE ;
}

patBoolean bioPythonWrapper::isHessianAvailable(patError*& err) const {
  return patFALSE ;
}

patBoolean bioPythonWrapper::isHessianTimesVectorAvailable(patError*& err) const {
  return patFALSE ;
}

unsigned long bioPythonWrapper::getDimension(patError*& err) const {
  err = new patErrMiscError("This function should never be called. It must be overloaded") ;
  WARNING(err->describe()) ;
  return 0 ;
}


trVector bioPythonWrapper::getCurrentVariables(patError*& err) const {
  err = new patErrMiscError("This function should never be called. It must be overloaded") ;
  WARNING(err->describe()) ;
  return trVector() ;
}

patBoolean bioPythonWrapper::isUserBased() const {
  return patFALSE ;
}

void bioPythonWrapper::setSample(bioSample* s,patError*& err) {
  sample = s ;
  if (sample != NULL) {
    patULong nThreads = bioParameters::the()->getValueInt("numberOfThreads",err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return ;
    }
    bioIteratorInfoRepository::the()->generateSublistsForThreads(sample->size(),nThreads,err) ;
  }
}


void bioPythonWrapper::generateCppCode(ostream& str, patError*& err) {
  err = new patErrMiscError("Not implemented") ;
  WARNING(err->describe()) ;
  return ;
}
