//-*-c++-*------------------------------------------------------------
//
// File name : bioSampleIterator.cc
// Author :    Michel Bierlaire and Mamy Fetiarison
// Date :      Mon Jul 13 15:27:42  2009
//
//--------------------------------------------------------------------

#include "patErrNullPointer.h"
#include "patErrMiscError.h"
#include "patDisplay.h"
#include "bioIteratorInfoRepository.h"
#include "bioRowIterator.h"
#include "patMath.h"

bioRowIterator::bioRowIterator(vector<patVariables>* db, 
			       bioIteratorSpan aSpan,
			       bioIteratorSpan threadSpan,
			       patError*& err) :
  dataBase(db),
  firstRow(aSpan.firstRow),
  lastRow(aSpan.lastRow) {

  iteratorInfo = bioIteratorInfoRepository::the()->getIteratorInfo(aSpan.name) ;

  if (iteratorInfo == NULL) {
    DEBUG_MESSAGE("--> name = " << aSpan.name)
    err = new patErrNullPointer("bioIteratorInfo") ;
    WARNING(err->describe()) ;
    return ;
  }
  rowPointers = &(iteratorInfo->rowPointers) ;
  
  if (iteratorInfo->isTopIterator()) {
    bioIteratorSpan intersect = aSpan.intersection(threadSpan) ;
    firstRow = intersect.firstRow ;
    lastRow = intersect.lastRow ;
  }


}


void bioRowIterator::first() {
  currentRow = firstRow ;
}


void bioRowIterator::next() {
  if (isDone()) {
    return ;
  }
  if (currentRow != patBadId) {
    if (currentRow > rowPointers->size()) {
      WARNING("ROW NUMBER OUT OF BOUNDS: " << currentRow <<  " > " << rowPointers->size()) ;
    }
    else {
      currentRow = (*rowPointers)[currentRow] ;
    }
  }
}


patBoolean bioRowIterator::isDone() {
  if (currentRow == patBadId) {
    return patTRUE ;
  }
  return (currentRow >= lastRow) ;
  
}


patVariables* bioRowIterator::currentItem() {
  if (!this->isDone()) {
    if (dataBase == NULL) {
      WARNING(" NO DATABASE") ;
      return NULL ;
    }
    if (currentRow >= dataBase->size()) {
      return NULL ;
    }
    return &((*dataBase)[currentRow]) ;
  } 
  else {
    return NULL ;
  }
}


